@@ -29,10 +29,14 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
+ *  Copyright (c) 1998-2008 The OpenSSL Project.  All rights reserved.
+ *  Copyright (c) 1995-1998 Eric Young (eay@cryptsoft.com)
+ *  All rights reserved.
+ *  
  *  The source code is derived from OpenSSL project.
  *  (See the following URL: http://www.openssl.org/source/license.html)
  *
- *  Last-modified: Mon 01 Dec 2008 21:49:00 JST
+ *  Last-modified: Sat 27 Dec 2008 05:39:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_SSL_H
@@ -57,7 +61,7 @@
 #include <openssl/err.h>
 #include <openssl/rand.h>
 
-#ifdef CLX_WIN32
+#if defined(_MSC_VER) && (_MSC_VER >= 1200)
 #pragma comment(lib, "libeay32.lib")
 #pragma comment(lib, "ssleay32.lib")
 #endif // CLX_WIN32
@@ -95,20 +99,23 @@ namespace clx {
 		static const int nbuf = 1024;
 		
 		explicit verify_error(int status) :
-			std::runtime_error("certification failed"), errno_(status) {}
+			std::runtime_error("certification failed"),
+			errno_(status), message_() {}
 		
 		virtual ~verify_error() throw() {}
 		
 		virtual const char_type* what() {
 			sstream_type tmp;
 			tmp << std::runtime_error::what() << ": status 0x" << std::hex << errno_;
-			return tmp.str().c_str();
+			message_ = tmp.str();
+			return message_.c_str();
 		}
 		
 		int code() const { return errno_; }
 		
 	private:
 		int errno_;
+		string_type message_;
 	};
 	
 	namespace ssl {
@@ -117,6 +124,7 @@ namespace clx {
 		/* ----------------------------------------------------------------- */
 		enum { sslv2 = 0, sslv3, tlsv1, sslv23 };
 		
+		template <int Version, int Family> class basic_acceptor;
 		/* ----------------------------------------------------------------- */
 		//  basic_socket
 		/* ----------------------------------------------------------------- */
@@ -125,6 +133,8 @@ namespace clx {
 			int Family
 		>
 		class basic_socket : public basic_rawsocket<SOCK_STREAM, Family, 0> {
+			friend class basic_acceptor<Version, Family>;
+			
 		public:
 			typedef basic_rawsocket<SOCK_STREAM, Family, 0> rawsocket;
 			typedef SSL handler;
@@ -137,23 +147,31 @@ namespace clx {
 			
 			basic_socket(const basic_socket& cp) : rawsocket(cp), p_(cp.p_), addr_(cp.addr_) {}
 			
+			template <class Socket>
+			basic_socket(const Socket& cp) :
+				rawsocket(cp), p_(), addr_(cp.address()) {
+				this->xinit(client);
+				p_->connect() = cp.is_connect();
+				this->xconnect();
+			}
+			
 			// for client ssl socket
 			explicit basic_socket(const char_type* host, int port) :
 				rawsocket(), p_(), addr_(host, port) {
-				this->ssl_init(client);
+				this->xinit(client);
 				this->xconnect();
 			}
 			
 			explicit basic_socket(const string_type& host, int port) :
 				rawsocket(), p_(), addr_(host, port) {
-				this->ssl_init(client);
+				this->xinit(client);
 				this->xconnect();
 			}
 			
 			// for server ssl socket
 			explicit basic_socket(socket_int s, const sockaddress& addr) :
 				rawsocket(s), p_(), addr_(addr) {
-				this->ssl_init(server);
+				this->xinit(server);
 				p_->connect() = true;
 			}
 			
@@ -178,7 +196,7 @@ namespace clx {
 			
 			virtual int send(const char_type* src, int n) {
 				if (p_.get() == NULL || p_->ssl() == NULL) return -1;
-				return ::SSL_write(p_->ssl(), src, n);
+				return SSL_write(p_->ssl(), src, n);
 			}
 			
 			virtual int send(const string_type& src) {
@@ -187,7 +205,7 @@ namespace clx {
 			
 			virtual int recv(char_type* dest, int n) {
 				if (p_.get() == NULL || p_->ssl() == NULL) return -1;
-				return ::SSL_read(p_->ssl(), dest, n);
+				return SSL_read(p_->ssl(), dest, n);
 			}
 			
 			virtual void close() {
@@ -197,11 +215,11 @@ namespace clx {
 			
 			virtual int pending() const {
 				if (p_.get() == NULL || p_->ssl() == NULL) return 0;
-				return ::SSL_pending(p_->ssl());
+				return SSL_pending(p_->ssl());
 			}
 			
 			bool verify_locations(const char_type* file, const char_type* dir = NULL) {
-				if (p_.get() == NULL || p_->ctx() == NULL) this->ssl_init(client);
+				if (p_.get() == NULL) this->xinit(client);
 				if (SSL_CTX_load_verify_locations(p_->ctx(), file, dir) == 0) return false;
 				p_->verify() = true;
 				return true;
@@ -214,7 +232,7 @@ namespace clx {
 				else return this->verify_locations(file.c_str(), dir.c_str());
 			}
 			
-			bool is_connect() const { return (p_.get() != NULL) ? p_->connect() : false; }
+			bool is_connect() const { return (p_.get() != NULL) ? p_->connect() && p_->handshake() : false; }
 			const sockaddress& address() const { return addr_; }
 			handler* ssl() { return (p_.get() != NULL) ? p_->ssl() : NULL; }
 			const handler* ssl() const { return (p_.get() != NULL) ? p_->ssl() : NULL; }
@@ -229,12 +247,13 @@ namespace clx {
 				typedef ctx_type* ctx_pointer;
 				typedef handler* handle_pointer;
 				
-				ssl_impl() : ctx_(NULL), ssl_(NULL), connect_(false), verify_(false) {}
+				ssl_impl() : ctx_(NULL), ssl_(NULL), connect_(false), handshake_(false), verify_(false) {}
 				~ssl_impl() { this->close(); }
 				
 				ctx_pointer& ctx() { return ctx_; }
 				handle_pointer& ssl() { return ssl_; }
 				bool& connect() { return connect_; }
+				bool& handshake() { return handshake_; }
 				bool& verify() { return verify_; }
 				
 				void close() {
@@ -250,6 +269,7 @@ namespace clx {
 					}
 					
 					connect_ = false;
+					handshake_ = false;
 					verify_ = false;
 				}
 				
@@ -257,13 +277,18 @@ namespace clx {
 				ctx_pointer ctx_;
 				handle_pointer ssl_;
 				bool connect_;
+				bool handshake_;
 				bool verify_;
 			};
 			
 			shared_ptr<ssl_impl> p_;
 			sockaddress addr_;
 			
-			void ssl_init(int mode) {
+			basic_socket& xinit(int mode) {
+				if (!this->is_open()) this->open();
+				if (!this->is_open()) throw socket_error("open");
+				if (p_.get() == NULL) p_ = shared_ptr<ssl_impl>(new ssl_impl());
+				
 				SSL_METHOD *(*f)() = NULL;
 				switch (Version) {
 				case sslv2:
@@ -285,14 +310,9 @@ namespace clx {
 					break;
 				}
 				
-				if (p_.get() == NULL) p_ = shared_ptr<ssl_impl>(new ssl_impl());
-				
 				if (p_->ctx() == NULL) p_->ctx() = SSL_CTX_new(f());
 				if (p_->ctx() == NULL) throw std::runtime_error("SSL_CTX_new failed");
 				
-				if (!this->is_open()) this->open();
-				if (!this->is_open()) throw socket_error("open");
-				
 				char err[1024];
 				if (p_->ssl() == NULL) p_->ssl() = SSL_new(p_->ctx());
 				if (p_->ssl() == NULL || SSL_set_fd(p_->ssl(), this->socket()) == 0) {
@@ -300,31 +320,37 @@ namespace clx {
 					ERR_error_string(ERR_get_error(), err);
 					throw std::runtime_error(err);
 				}
+				
+				return *this;
 			}
 			
 			// for client ssl socket
 			basic_socket& xconnect() {
-				if (p_.get() == NULL || p_->ctx() == NULL || p_->ssl() == NULL) this->ssl_init(client);
-				else if (p_->connect()) return *this;
+				if (p_.get() == NULL) this->xinit(client);
 				
-				if (::connect(this->socket(), (struct sockaddr*)addr_.data(), addr_.size()) == -1) {
-					throw clx::socket_error("connect");
-				}
-				p_->connect() = true;
-				
-				RAND_poll();
-				mt19937 engine(static_cast<unsigned long>(std::time(NULL)));
-				random<unsigned short> rng(engine);
-				while (RAND_status() == 0) {
-					unsigned short value = rng() % 65536;
-					RAND_seed(&value, sizeof(value));
+				if (!p_->connect()) {
+					if (::connect(this->socket(), (struct sockaddr*)addr_.data(), addr_.size()) == -1) {
+						throw clx::socket_error("connect");
+					}
+					p_->connect() = true;
 				}
 				
-				char err[1024];
-				if (SSL_connect(p_->ssl()) <= 0) {
-					std::memset(err, '\0', sizeof(err));
-					ERR_error_string(ERR_get_error(), err);
-					throw std::runtime_error(err);
+				if (!p_->handshake()) {
+					RAND_poll();
+					mt19937 engine(static_cast<unsigned long>(std::time(NULL)));
+					random<unsigned short> rng(engine);
+					while (RAND_status() == 0) {
+						unsigned short value = rng() % 65536;
+						RAND_seed(&value, sizeof(value));
+					}
+					
+					char err[1024];
+					if (SSL_connect(p_->ssl()) <= 0) {
+						std::memset(err, '\0', sizeof(err));
+						ERR_error_string(ERR_get_error(), err);
+						throw std::runtime_error(err);
+					}
+					p_->handshake() = true;
 				}
 				
 				if (p_->verify()) {
@@ -348,7 +374,7 @@ namespace clx {
 		/* ----------------------------------------------------------------- */
 		template <
 			int Version,
-			int Family = AF_INET
+			int Family
 		>
 		class basic_acceptor : public basic_rawsocket<SOCK_STREAM, Family> {
 		public:
@@ -403,6 +429,7 @@ namespace clx {
 					ERR_error_string(ERR_get_error(), err);
 					throw std::runtime_error(err);
 				}
+				client.p_->handshake() = true;
 				
 				return client;
 			}
