@@ -40,6 +40,7 @@
 #define CLX_THREAD_H
 
 #ifndef CLX_WIN32
+//#if defined(_MSC_VER) && (_MSC_VER >= 1200)
 #if defined(_WIN32) || defined(__WIN32__) || defined(WIN32)
 #define CLX_WIN32
 #endif
@@ -47,7 +48,6 @@
 
 #include "shared_ptr.h"
 #include "timer.h"
-#include "time_duration.h"
 #include "mutex.h"
 
 #ifdef CLX_WIN32
